--[[ borlox 2009-04-30T18:06:55Z f86a59b ]]--
local CCopy = LibStub("AceAddon-3.0"):NewAddon("CCopy", "AceTimer-3.0", "AceHook-3.0")

---------------------------------------------------------------------
---- Configuration

-- Every x seconds the old lines will be purged
local CLEANUP_DELAY = 600 -- 10 minutes

-- Number of messages that should be kept while purging
local KEEP_NUM_MESSAGES = 20

-- This string is displayed before saved chat lines
local MARKER = '*'

-- Show the time instead of the marker above
local SHOW_TIME = false

-- Format string for the time (see http://www.opengroup.org/onlinepubs/007908799/xsh/strftime.html)
local TIME_FMT = "[%H:%M:%S]"

---- You should not touch anything beyond this line...
---------------------------------------------------------------------

local link_msg = "|HCCopy:%d|h%s|h %s"
local find_id  = "^CCopy:(%d+)"

local ChatFrameEditBox = ChatFrameEditBox

function CCopy:OnInitialize()
	self.messages = {} -- this table is getting really! big after some time, so it's cleared out every 10 minutes
end

function CCopy:OnEnable()
	self:RawHook("SetItemRef", true)
	local _G = _G
	for i=1, NUM_CHAT_WINDOWS do
		self:RawHook(_G["ChatFrame"..i], "AddMessage", true)
	end
	self:ScheduleRepeatingTimer("CleanUp", CLEANUP_DELAY)
end

function CCopy:ClearMessage(msg)
	msg = msg:gsub("|H.-|h(.-)|h", "%1")	-- clear links
	msg = msg:gsub("|c%x%x%x%x%x%x%x%x(.-)|r", "%1") -- clear colors
	return msg
end

function CCopy:ShowMessage(id)
	local msg = self.messages[id]
	if not msg then 
		msg = "<This line was allready deleted from the log. Sorry>"
	end
	msg = self:ClearMessage(msg)
	
	ChatFrameEditBox:SetText(ChatFrameEditBox:GetText() .. msg)
	if not ChatFrameEditBox:IsShown() then
		ChatFrameEditBox:Show()
		ChatFrameEditBox:HighlightText()
	end
end

function CCopy:AddMessage(chatFrame, msg, ...)
	local id = #self.messages + 1
	self.messages[id] = msg
	msg = link_msg:format(id, SHOW_TIME and date(TIME_FMT) or MARKER, msg)
	self.hooks[chatFrame].AddMessage(chatFrame, msg, ...)
end

function CCopy:SetItemRef(link, ...)
	if link:sub(1, 5) == "CCopy" then
		local _,_,id = link:find(find_id)
		self:ShowMessage(tonumber(id))
	else
		self.hooks.SetItemRef(link, ...)
	end
end

function CCopy:CleanUp()
	local messages = self.messages
	local newMessages = {}
	for i=(#messages - KEEP_NUM_MESSAGES), #messages do
		newMessages[i] = messages[i]
	end
	self.messages = newMessages
	if not InCombatLockdown() then
		collectgarbage("collect")
	end
end
